"""
@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil

from packetlogic2.exceptions import PLDBUnsupportedInFirmware

def _verify_pos(aspath_id):
	if aspath_id < -1 or aspath_id > 255:
		return False
	return True

class BGPObject(plobject.PLObject):
	"""
	BGPObject Implementation

	Object path: /BGPObjects

	@cvar Item: BGPObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	
	base_path = plutil.roConst("/BGPObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_BGPOBJECT, "Number representing the type of object")

	def _create_item(self, value):
		if not isinstance(value, basestring):
			raise TypeError("Item must be a string formatted as 'asnumber@pos' or 'asnumber:community'")

		if value.find("@") != -1:
			try:
				asnumber, pos = value.split("@", 1)
				item = self.Item(self, asnumber=asnumber, pos=pos)
			except:
				raise ValueError("Not in the correct format, must be asnumber@pos")

			if not _verify_pos(item._prop.pos):
				raise ValueError("Position not a valid value")

		elif value.find(":") != -1:
			raise PLDBUnsupportedInFirmware("Usage of 'asnumber:community' is not supported in this firmware version")
		else:
			raise ValueError("Item must be a string formatted as 'asnumber@pos' or 'asnumber:community'")

		return item

	def add(self, value):
		"""
		Adds a BGPObject item to this BGPObject.

		@type value: C{str}

		@param value: This parameter describes the value of
		              the BGPObject item.  The string is
		              formatted as 'asnumber@position' or
		              'asnumber:community'.  Position must be
		              between -1 and 255. -1 = Last pos, 0 =
		              Any pos.

		@attention: The 'asnumber:community' format requires
		            firmware version 13.1.9 or newer.
		"""
		plobject.PLObject.add(self, value)


	class Item(plobject.PLObject.Item):
		"""
		BGPObject Item implementation.
		Should not be instantiated, use L{BGPObject.add} instead.

		@type type: C{int}
		@type asnumber: C{int}
		@type value1: C{int}
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(asnumber=int, pos=int, *plobject._item_props)

		def __str__(self):
			return "%d@%d" % (self._prop.asnumber, self._prop.pos)

		type = property(lambda self: 0, doc="Type (0=ASNumber@Pos, 1=ASregexp, 2=ASNumber:Community)")
		asnumber = property(lambda self: self._prop.asnumber, doc="ASNumber")
		pos = property(lambda self: self._prop.pos, doc="Position (-1 to 255)")
		value1 = property(lambda self: self._prop.pos, doc="Position (-1 to 255) or Community number")
